#include "Parsodus/lrtables/LR0Itemset.h"

namespace pds {
namespace lr {

LR0Itemset::LR0Itemset() {
}

LR0Itemset::LR0Itemset(std::shared_ptr<Rule> start) {
    m_items.emplace(LR0Item{start, 0});
}

void LR0Itemset::close(const Grammar& g) {
    bool changes = true;
    std::vector<LR0Item> todo;
    std::set<std::string> added;

    while (changes) {
        changes = false;
        std::set<LR0Item> toAdd;

        for (const LR0Item& i : m_items) {
            if (i.dotIdx < i.rule->tail.size()) {
                std::string& sym = i.rule->tail[i.dotIdx];
                if (g.variables.count(sym) && !added.count(sym)) {
                    added.insert(sym);
                    changes = true;
                    for (const auto& rule : g.rules) {
                        if (rule->head == sym) {
                            toAdd.insert(LR0Item{rule, 0});
                        }
                    }
                }
            }
        }
        m_items.insert(toAdd.begin(), toAdd.end());
    }
}

LR0Itemset LR0Itemset::succ(std::string sym) const {
    LR0Itemset sc;
    for (auto& item : m_items) {
        if (item.dotIdx < item.rule->tail.size()) {
            if (item.rule->tail[item.dotIdx] == sym) {
                sc.m_items.insert(LR0Item{item.rule, item.dotIdx + 1});
            }
        }
    }
    return sc;
}

bool LR0Itemset::operator==(const LR0Itemset& rhs) const {
    return m_items == rhs.m_items;
}

bool LR0Itemset::canMerge(const LR0Itemset&) const {
    return false;
}

void LR0Itemset::merge(const LR0Itemset&) {
    //NO-OP
}

bool LR0Itemset::empty() const {
    return m_items.empty();
}

std::set<std::size_t> LR0Itemset::getReduces(const Grammar& g, std::string) const {
    std::set<std::size_t> res;
    for (auto& item : m_items) {
        if (item.dotIdx >= item.rule->tail.size()) {
            res.insert(std::find(g.rules.begin(), g.rules.end(), item.rule) - g.rules.begin());
        }
    }
    return res;
}

} /* lr  */ 
} /* pds  */ 
